SystemOrganization addCategory: #'SqueakNxt-Compiler-Programs'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Translation'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-TranslationViejo'!
SystemOrganization addCategory: #'SqueakNxt-Compiler-Api'!

!String methodsFor: '*SqueakNxt-Compiler' stamp: 'Richo 1/1/2011 15:42'!
asNxcRule
	^ NxcPrimitiveRule on: self ! !

Object subclass: #Nbc
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!Nbc commentStamp: 'Richo 12/25/2010 01:12' prior: 0!
A NbcExe is a very simple API for accessing the nbc compiler using FFI on Windows

Examples: 
1) The simplest example would be compiling into a file called "output" and reporting errors to a file called "errors":

	NbcExe new
		fileName: 'example_program.nbc';
		outFile: 'output';
		errorsFile: 'errors';
		execute.
		
2) To download a program into the Lego Nxt using the usb:

	NbcExe new
		fileName: 'example_program.nbc';
		portName: 'usb';
		downloadProgram;
		execute.
		
3) To use the bluetooth on COM6 instead:
	
	NbcExe new
		fileName: 'example_program.nbc';
		useBluetooth;
		portName: 'COM6';
		downloadProgram;
		execute.

Instance Variables
	parameters:		<WriteStream>

parameters
	- Used to build the parameters string to be sent to the nbc program. See "nbc -help" for more information.
!

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
beQuiet
	parameters space; nextPutAll: '-q'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
decompileProgram
	parameters space; nextPutAll: '-x'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
defineMacro: aByteString 
	parameters space; nextPutAll: '-D='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontCompile
	parameters space; nextPutAll: '-b'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontIncludeSystemFile
	parameters space; nextPutAll: '-n'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
downloadProgram
	parameters space; nextPutAll: '-d'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:25'!
enhancedFirmware
	parameters space; nextPutAll: '-EF'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
errorsFile: aByteString 
	parameters space; nextPutAll: '-E='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'executing' stamp: 'Richo 12/29/2010 17:57'!
execute
	"Executes the command using FFI. Windows only for now"
	| result shell |
	shell := Smalltalk at: #Win32Shell ifAbsent: [^self error: 'FFI doesn''t seem to be installed'].
	(result := (shell new 
		shellExecute:  nil
		lpOperation:  'open'
		lpFile:  'nbc.exe'
		lpParameters:  parameters contents
		lpDirectory:  nil
		nShowCmd: 0) <= 32) ifTrue: [self error: 'system error, code:', result]! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
fileName: aByteString 
	parameters space; nextPutAll: aByteString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
includePath: aByteString 
	parameters space; nextPutAll: '-I='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'initialize' stamp: 'Richo 12/25/2010 00:44'!
initialize
	super initialize.
	parameters := WriteStream on: ''! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/29/2010 17:59'!
nbc: fileName
	parameters space; nextPutAll: '-nbc='; nextPutAll: fileName! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
outFile: aByteString 
	parameters space; nextPutAll: '-O='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
portName: aByteString 
	parameters space; nextPutAll: '-S='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:48'!
useBluetooth
	parameters space; nextPutAll: '-BT'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/28/2010 18:38'!
warningsOn
	parameters space; nextPutAll: '-w'! !

Object subclass: #NbcCodeBlock
	instanceVariableNames: 'statements name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcCodeBlock class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 17:46'!
name: aString statements: statements
	^ (self basicNew setName: aString statements: statements) initialize! !

!NbcCodeBlock methodsFor: 'printing' stamp: 'Richo 12/28/2010 17:48'!
printOn: aStream
aStream nextPutAll: name; cr.
statements do: [:each | aStream tab.
	each printOn: aStream.
	aStream cr].! !

!NbcCodeBlock methodsFor: 'private' stamp: 'Richo 12/28/2010 17:47'!
setName: aString statements: anOrderedCollection 
	name := aString.
	statements := anOrderedCollection ! !

NbcCodeBlock subclass: #NbcSubroutine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcSubroutine methodsFor: 'printing' stamp: 'Richo 12/28/2010 17:49'!
printOn: aStream
aStream nextPutAll: 'subroutine'; space.
super printOn: aStream.
aStream nextPutAll: 'return'; cr; nextPutAll: 'ends'! !

NbcCodeBlock subclass: #NbcThread
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcThread methodsFor: 'printing' stamp: 'Richo 12/28/2010 17:46'!
printOn: aStream
aStream nextPutAll: 'thread'; space.
super printOn: aStream.
aStream nextPutAll: 'endt'! !

Object subclass: #NbcProgram
	instanceVariableNames: 'variables codeBlocks name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcProgram class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 17:02'!
variables: aSet codeBlocks: anArray 
	^ (self basicNew setVariables: aSet codeBlocks: anArray ) initialize ! !

!NbcProgram methodsFor: 'compiling' stamp: 'Richo 12/31/2010 21:46'!
compileInFileNamed: fileName
	^ NxcCompiler new program: self; compileInFileNamed: fileName! !

!NbcProgram methodsFor: 'compiling' stamp: 'Richo 12/31/2010 21:46'!
downloadUsingBluetooth: portName
	^ NxcCompiler new program: self; downloadUsingBluetooth: portName ! !

!NbcProgram methodsFor: 'compiling' stamp: 'Richo 12/31/2010 21:46'!
downloadUsingUsb
	^ NxcCompiler new program: self; downloadUsingUsb! !

!NbcProgram methodsFor: 'accessing' stamp: 'Richo 12/29/2010 15:05'!
name
	^ name! !

!NbcProgram methodsFor: 'accessing' stamp: 'Richo 12/29/2010 15:05'!
name: aString
	name := aString ! !

!NbcProgram methodsFor: 'printing' stamp: 'Richo 12/28/2010 17:48'!
printOn: aStream
aStream nextPutAll: 'dseg segment'; cr.
variables do: [:each|
	each = 'self' ifFalse: [aStream tab; nextPutAll: each; space; nextPutAll: 'slong'; cr].
	].
aStream nextPutAll: 'dseg ends'; cr; cr.
codeBlocks do: [:each | each printOn: aStream. aStream cr; cr]! !

!NbcProgram methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 03:00'!
setName: aByteSymbol source: aWriteStream 
	self shouldBeImplemented! !

!NbcProgram methodsFor: 'private' stamp: 'Richo 12/28/2010 17:02'!
setVariables: aSet codeBlocks: anArray 
	variables := aSet.
	codeBlocks := anArray.! !

!NbcProgram methodsFor: 'compiling' stamp: 'Richo 12/31/2010 21:46'!
verify
	^ NxcCompiler new program: self; verify! !

Object subclass: #NbcStatement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

NbcStatement subclass: #NbcAssignmentStatement
	instanceVariableNames: 'variable value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcAssignmentStatement class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 15:32'!
variable: aByteString value: aByteString2 
^( self basicNew setVariable: aByteString value: aByteString2) initialize ! !

!NbcAssignmentStatement methodsFor: 'printing' stamp: 'Richo 12/28/2010 15:34'!
printOn: aStream
aStream nextPutAll: 'mov'; space; nextPutAll: variable; nextPut: $,; space; nextPutAll: value! !

!NbcAssignmentStatement methodsFor: 'private' stamp: 'Richo 12/28/2010 15:32'!
setVariable: aByteString value: aByteString2 
	variable := aByteString.
	value := aByteString2 ! !

NbcStatement subclass: #NbcMacroFunctionCall
	instanceVariableNames: 'arguments rule'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcMacroFunctionCall class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 15:37'!
rule: aByteSymbol arguments: anArray 
	^ (self basicNew setRule: aByteSymbol arguments: anArray ) initialize ! !

!NbcMacroFunctionCall methodsFor: 'printing' stamp: 'Richo 12/28/2010 19:13'!
printOn: aStream
| tempRule |
tempRule := rule.
arguments doWithIndex: [:arg :i | 
tempRule := tempRule copyReplaceAll: 'arg', (i - 1) asString with: arg].
aStream nextPutAll: tempRule.! !

!NbcMacroFunctionCall methodsFor: 'private' stamp: 'Richo 12/28/2010 15:38'!
setRule: aByteSymbol arguments: anArray 
	rule := aByteSymbol.
	arguments := anArray ! !

NbcStatement subclass: #NbcMathStatement
	instanceVariableNames: 'variable arguments name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcMathStatement class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 15:35'!
name: aByteSymbol variable: aByteString arguments: anArray 
	^ (self basicNew setName: aByteSymbol variable: aByteString arguments: anArray ) initialize! !

!NbcMathStatement methodsFor: 'printing' stamp: 'Richo 12/28/2010 15:36'!
printOn: aStream
aStream nextPutAll: name; space;
		nextPutAll: variable; nextPut: $,; space.
arguments do: [:arg | aStream nextPutAll: arg] separatedBy: [aStream nextPut: $,; space]! !

!NbcMathStatement methodsFor: 'private' stamp: 'Richo 12/28/2010 15:36'!
setName: aByteSymbol variable: aByteString arguments: anArray 
	name := aByteSymbol.
	variable := aByteString.
	arguments := anArray ! !

NbcStatement subclass: #NbcTimingStatement
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcTimingStatement class methodsFor: 'instance creation' stamp: 'Richo 12/28/2010 16:52'!
value: aByteString 
	^ (self basicNew setValue: aByteString) initialize ! !

!NbcTimingStatement methodsFor: 'printing' stamp: 'Richo 12/28/2010 16:53'!
printOn: aStream
aStream nextPutAll: 'waitv'; space; nextPutAll: value! !

!NbcTimingStatement methodsFor: 'private' stamp: 'Richo 12/28/2010 16:52'!
setValue: aByteString 
	value := aByteString ! !

Object subclass: #NxcCompiler
	instanceVariableNames: 'programName source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcCompiler commentStamp: 'Richo 12/31/2010 22:03' prior: 0!
A NxcCompiler is responsible for verifying/compiling programs as well as downloading them to the LegoNxt.

===============================
	"Code example"
	
	| program |
	
	"A simple nxc program"
	program := '
	#define MOVE_TIME   1000
	#define TURN_TIME    500

	task main()
	{
		repeat(4)
		{    
			OnFwd(OUT_AC, 75);
			Wait(MOVE_TIME);
			OnRev(OUT_C, 75);
		Wait(TURN_TIME);
		}  
		Off(OUT_AC);
	}'.
	
	"The code below will verify the program and return a collection of errors (if any)"
	NxcCompiler new
		source: program;
		verify.
	
	"The code below will download the program to the nxt using the usb cable. the resulting program will be called 'SimpleProgram'"
	NxcCompiler new
		programName: 'SimpleProgram';
		source: program;
		downloadUsingUsb.
===============================

Instance Variables
	programName:		<String>
	source:		<String>

programName
	- A name for the program. If not specified the program name will be the NxcCompiler instance's hash (not a nice name for a program, I must say).

source
	- The nxc source code of the program.
!

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
compileInFileNamed: outFile
	"Compiles the program into a file"
	^ self execute: (Nbc new outFile: outFile)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:10'!
downloadUsingBluetooth: portName 
	"Compiles the program and downloads it using the bluetooth on the specified port name (COMn). Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: portName; useBluetooth; downloadProgram)! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:09'!
downloadUsingUsb
	"Compiles the program and downloads it using the usb cable. Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: 'usb'; downloadProgram)! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:14'!
errorsFileName
	"Temporary file name to store errors during compilation"
	^ self programName , '.err'! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/28/2010 18:38'!
execute: nbc 
	"Create the temporary files to store the source code and the errors.
	Execute nbc.
	Wait until nbc finishes and parse errors.
	Delete temporary files.
	Return errors"
	source isNil ifTrue: [^ self error: 'Source cannot be nil'].
	FileStream forceNewFileNamed: self programFileName do: [:file | file nextPutAll: source]; 
				forceNewFileNamed: self errorsFileName do: [:file | "Do nothing. Nbc will write this file"].
	[nbc fileName: self programFileName;
		 errorsFile: self errorsFileName;
		 execute.
	^ self parseErrors]
		ensure: [FileDirectory default deleteFileNamed: self programFileName;
				 deleteFileNamed: self errorsFileName]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:12'!
parse: errorsFile
"Returns a collection with all the errors founded by the nbc compiler. The errorsFile format is similar to this:

# Error: Unknown or invalid statement
File ""D:\RICHI\Squeak images\Nxt\simpleprogram.nbc"" ; line 7
#   result sbyt
#----------------------------------------------------------
1 error during compilation
"
| errors |
errors := OrderedCollection new.
[errorsFile peek = $#] 
	whileTrue: [ | errorDescription errorPosition errorSource |
		errorDescription := errorsFile nextLine allButFirst: 2.
		errorPosition := (errorsFile nextLine subStrings: {$;}) last withoutLeadingBlanks.
		errorSource := errorsFile nextLine allButFirst: 4.
		errorsFile nextLine.
		errors add: (NbcError description: errorDescription position: errorPosition source: errorSource).
		].
^errors asArray! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 22:05'!
parseErrors
	"Wait until the errors file is fully written by nbc, then read it and return the errors.
	IMPORTANT: Be aware that this code relies on the fact that FileStream>>#fileNamed: returns nil when the file is being modified by another process. If this somehow changes in the future, this code will inevitably break."
	| errorsFile |
	[500 milliSeconds asDelay wait.
	(errorsFile := FileStream fileNamed: self errorsFileName) isNil] whileTrue.
	[^ self parse: errorsFile]
		ensure: [errorsFile close].
! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
program: aNxcProgram
	source := aNxcProgram source.
	programName isNil ifTrue: [programName := aNxcProgram name]! !

!NxcCompiler methodsFor: 'private' stamp: 'Richo 12/29/2010 17:43'!
programFileName
	"Temporary file name to store the source code"
	^ self programName , '.nxc'! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 22:10'!
programName
	"If programName is not specified, the default name will be this instance's hash"
	^ programName ifNil: [self hash asString]! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:47'!
programName: aString
	programName := aString! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 17:07'!
source
	^ source! !

!NxcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:48'!
source: aString
	source := aString.! !

!NxcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 22:10'!
verify
	"Compiles the program and returns a collection of NbcErrors. If the collection is empty then the program is ok"
	^ self execute: Nbc new! !

Object subclass: #NxcError
	instanceVariableNames: 'description position source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Api'!

!NxcError class methodsFor: 'instance creation' stamp: 'Richo 12/25/2010 16:02'!
description: description position: position source: source
^ (self basicNew setDescription: description position: position source: source) initialize ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
description
	^ description ! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
position
	^ position ! !

!NxcError methodsFor: 'printing' stamp: 'Richo 12/27/2010 22:08'!
printOn: aStream
aStream nextPutAll: position asString;
		 nextPut: $:;
		 space;
		 nextPutAll: description asString 
! !

!NxcError methodsFor: 'private' stamp: 'Richo 12/25/2010 16:02'!
setDescription: aByteString position: aByteString2 source: aByteString3 
	description := aByteString.
	position := aByteString2.
	source := aByteString3! !

!NxcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
source
	^ source ! !

Object subclass: #NxcProgram
	instanceVariableNames: 'name source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!NxcProgram class methodsFor: 'instance creation' stamp: 'Richo 1/1/2011 03:01'!
name: programName source: programSource
	^ (self basicNew setName: programName source: programSource) initialize ! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
compileInFileNamed: fileName
	^ NxcCompiler new program: self; compileInFileNamed: fileName! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
downloadUsingBluetooth: portName
	^ NxcCompiler new program: self; downloadUsingBluetooth: portName ! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 15:33'!
downloadUsingUsb
	^ NxcCompiler new program: self; downloadUsingUsb! !

!NxcProgram methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
name
	^ name! !

!NxcProgram methodsFor: 'printing' stamp: 'Richo 1/1/2011 03:03'!
printOn: aStream
aStream nextPutAll: source.! !

!NxcProgram methodsFor: 'private' stamp: 'Richo 1/1/2011 03:02'!
setName: aByteSymbol source: aWriteStream 
	name := aByteSymbol.
	source := aWriteStream contents.! !

!NxcProgram methodsFor: 'accessing' stamp: 'Richo 1/1/2011 03:07'!
source
	^ source! !

!NxcProgram methodsFor: 'compiling' stamp: 'Richo 1/1/2011 03:02'!
verify
	^ NxcCompiler new program: self; verify! !

Object subclass: #NxtProgram
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 12/29/2010 15:06'!
asNbcProgram
^ self asNbcProgram: self name! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 12/29/2010 15:06'!
asNbcProgram: programName
^  (NbcTranslator new translate: self) name: programName ! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 1/1/2011 02:37'!
asNxcProgram
	^ self asNxcProgram: self name! !

!NxtProgram class methodsFor: 'translating' stamp: 'Richo 1/1/2011 02:37'!
asNxcProgram: programName 
	^ NxcTranslator new translate: self asProgramNamed: programName! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:20'!
definition
	"Answer a String that defines the receiver."
	"Overwritten to show the #instanceVariablesTypes: message in the class definition of NxtProgram subclasses"
	| aStream path |
	self = NxtProgram ifTrue: [^super definition].
	aStream := WriteStream
				on: (String new: 300).
	superclass == nil
		ifTrue: [aStream nextPutAll: 'ProtoObject']
		ifFalse: [path := ''.
			self environment
				scopeFor: superclass name
				from: nil
				envtAndPathIfFound: [:envt :remotePath | path := remotePath].
			aStream nextPutAll: path , superclass name].
	aStream nextPutAll: self kindOfSubclass;
		 store: self name.
	(self hasTraitComposition
			and: [self traitComposition notEmpty])
		ifTrue: [aStream cr; tab; nextPutAll: 'uses: '; nextPutAll: self traitCompositionString].
	aStream cr; tab; nextPutAll: 'instanceVariableNames: '; store: self instanceVariablesString.
	aStream cr; tab; nextPutAll: 'instanceVariableTypes: '; store: self instanceVariableTypesString.
	aStream cr; tab; nextPutAll: 'classVariableNames: '; store: self classVariablesString.
	aStream cr; tab; nextPutAll: 'poolDictionaries: '; store: self sharedPoolsString.
	aStream cr; tab; nextPutAll: 'category: '; store: (SystemOrganization categoryOfElement: self name) asString.
	superclass
		ifNil: [aStream nextPutAll: '.';
				 cr.
			aStream nextPutAll: self name.
			aStream space; nextPutAll: 'superclass: nil'].
	^ aStream contents! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:16'!
instVarTypes 
	| instVarTypes |
	instVarTypes := (self instVarTypesString withBlanksTrimmed subStrings: {Character space}) collect: [:each | each asSymbol].
	instVarTypes isEmpty ifTrue: [instVarTypes := #()].
	^ instVarTypes ! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:29'!
instanceVariableTypes: aString 
"Automatically compiles a method called #instanceVariableTypesString that returns the string given as a parameter"
self class compile: 'instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you''re doing"
^ ', aString printString, ' withBlanksCondensed' classified: #definition
! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:24'!
instanceVariableTypesString
	"This method will be automaticaly generated for subclasses whenever a subclass of NxtProgram is compiled. It returns the information of the inst vars' type. I choose to do it this way (instead of using an class inst var) to ensure the type info will be saved when we file out this class or store it using Monticello"
	^ ''! !

!NxtProgram class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:25'!
subclass: nameOfClass instanceVariableNames: instVarNames instanceVariableTypes: instVarTypes classVariableNames: classVarNames poolDictionaries: poolDictnames category: category
"Overwritten to allow defining the inst vars' type info of a class"
	^(ClassBuilder new
		superclass: self
		subclass: nameOfClass
		instanceVariableNames: instVarNames
		classVariableNames: classVarNames
		poolDictionaries: poolDictnames
		category: category) instanceVariableTypes: instVarTypes! !

!NxtProgram methodsFor: 'as yet unclassified' stamp: 'Richo 12/28/2010 17:25'!
displayNumber: var x: x y: y! !

!NxtProgram methodsFor: 'as yet unclassified' stamp: 'Richo 12/28/2010 17:25'!
onFwd: outputConstant speed: speed! !

!NxtProgram methodsFor: 'as yet unclassified' stamp: 'Richo 12/28/2010 17:25'!
onRev: outputConstant speed: speed! !

!NxtProgram methodsFor: 'as yet unclassified' stamp: 'Richo 12/28/2010 17:25'!
wait: milliseconds! !

NxtProgram subclass: #NxtProgramTest1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest1 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:22'!
anotherMethod
	self displayNumber: 100 x: 10 y: 50 - (10*5) ! !

!NxtProgramTest1 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:23'!
main
	self onFwd: 0 speed: 100;
		wait: 1000.! !

NxtProgram subclass: #NxtProgramTest2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest2 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest2 methodsFor: 'as yet unclassified' stamp: 'Richo 12/29/2010 14:19'!
main
	| var |
	var := 10 + 20 + (30 * 40).
	self displayNumber: var x: 10 y: 8;
		wait: var;
		onFwd: 0 speed: 100! !

NxtProgram subclass: #NxtProgramTest3
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest3 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'byte   char' withBlanksCondensed! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
func: arg1 and: arg2
	<argTypes: 'long long'>
	<tempTypes: 'long long'>
	<returnType: 'byte'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest3 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:59'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest4
	instanceVariableNames: 'instVar1 instVar2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest4 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ 'long long' withBlanksCondensed! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:55'!
func: arg1 and: arg2
	<argTypes: 'long'>
	<returnType: 'int'>
	| temp1 instVar2 |
	instVar2 := instVar1 .
	temp1 := instVar1.
	instVar1 := temp1 * instVar2.
	^ arg1 + arg2 
! !

!NxtProgramTest4 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 13:56'!
main
	<tempTypes: 'long'>
	| tempVar1 |
	tempVar1 := self func: 3 and: instVar1.
	instVar2 := 4.
	self displayNumber: tempVar1 * instVar2 x: 0 y: 0! !

NxtProgram subclass: #NxtProgramTest5
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Programs'!

!NxtProgramTest5 class methodsFor: 'definition' stamp: 'Richo 1/1/2011 16:30'!
instanceVariableTypesString
"This method was autogenerated. Do not change unless you *really* know what you're doing"
^ '' withBlanksCondensed! !

!NxtProgramTest5 methodsFor: 'as yet unclassified' stamp: 'Richo 1/1/2011 14:07'!
main
	
	5 = true
		ifTrue: [self wait: 1000.
			self displayNumber: 500 x: 0 y: 0]
		ifFalse: [self wait: 1200; displayNumber: 500 x: 0 y: 0].
		
	false ifTrue: [self onFwd: 0 speed: 100].
	true ifFalse: [self onFwd: 0 speed: -100]! !

Object subclass: #VariableInfo
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!VariableInfo methodsFor: 'as yet unclassified' stamp: 'Richo 12/31/2010 23:51'!
at: selector
	(variables includesKey: selector) ifFalse: [variables at: selector put: Set new].
	^ variables at: selector ! !

!VariableInfo methodsFor: 'as yet unclassified' stamp: 'Richo 12/31/2010 22:21'!
initialize
	super initialize.
	variables := Dictionary new! !

!VariableInfo methodsFor: 'as yet unclassified' stamp: 'Richo 12/31/2010 22:22'!
instVars
	^ self at: #instVars! !

!VariableInfo methodsFor: 'as yet unclassified' stamp: 'Richo 12/31/2010 23:49'!
isAnInstVar: varName
	^ (self instVars detect: [:each | each name = varName] ifNone: [nil]) notNil ! !

!VariableInfo methodsFor: 'accessing' stamp: 'Richo 1/1/2011 02:58'!
typeOf: var in: method
	^ ((self at: method) detect: [:each | each key = var]) value! !

RBProgramNodeVisitor subclass: #NbcTranslator
	instanceVariableNames: 'statements variables codeBlocks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-TranslationViejo'!

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 15:24'!
acceptAssignmentNode: anAssignmentNode
| value variable |
variable := self visitNode: anAssignmentNode variable.
value := self visitNode: anAssignmentNode value.
statements add: (NbcAssignmentStatement variable: variable value: value).
^ variable! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 15:06'!
acceptLiteralNode: aLiteralNode
	^aLiteralNode value asString! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 15:22'!
acceptMessageNode: aMessageNode
^ self createStatementFrom: aMessageNode! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 16:30'!
acceptVariableNode: aVariableNode
	^ variables add: aVariableNode name
	! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 17:28'!
createMacroFunctionFrom: aMessageNode 
	| arguments receiver selector |
	selector := aMessageNode selector.
	receiver := self visitNode: aMessageNode receiver.
	arguments := aMessageNode arguments collect: [:each | self visitNode: each].
	statements add: (NbcMacroFunctionCall rule: (self macroFunctions at: selector) arguments: {receiver} , arguments)! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 15:35'!
createMathStatementFrom: aMessageNode 
| argument receiver selector variable |
selector := aMessageNode selector.
variable := self newVariable.
receiver := self visitNode: aMessageNode receiver.
argument := self visitNode: (aMessageNode arguments first).
statements add: (NbcMathStatement name: (self mathStatements at: selector) variable: variable arguments: {receiver. argument}). 
^ variable.
! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 16:50'!
createStatementFrom: aMessageNode 
(self macroFunctions includesKey: aMessageNode selector)
	ifTrue: [^ self createMacroFunctionFrom: aMessageNode].
(self mathStatements includesKey: aMessageNode selector)
	ifTrue: [^ self createMathStatementFrom: aMessageNode].
(self timingStatements includesKey: aMessageNode selector)
	ifTrue: [^ self createTimingStatementFrom: aMessageNode].
self error: 'What the fuck?'! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 16:51'!
createTimingStatementFrom: aMessageNode 
	statements add: (NbcTimingStatement value: (self visitNode: aMessageNode arguments first)).! !

!NbcTranslator methodsFor: 'accessing' stamp: 'Richo 12/28/2010 17:24'!
macroFunctions
	^ {
	#displayNumber:x:y: -> 'NumOut(arg2, arg3, arg1)'.
	#onFwd:speed: -> 'OnFwd(arg1, arg2)'
	} as: Dictionary! !

!NbcTranslator methodsFor: 'accessing' stamp: 'Richo 12/28/2010 17:30'!
mathStatements
^ {
	#+ -> 'add'.
	#- -> 'sub'.
	#* -> 'mul'
} as: Dictionary ! !

!NbcTranslator methodsFor: 'visiting' stamp: 'Richo 12/28/2010 16:47'!
newVariable
| auxVarName |
auxVarName := 't', self hash asString.
^variables add: (auxVarName , (variables select: [:each | auxVarName, '*' match: each]) size asString).! !

!NbcTranslator methodsFor: 'accessing' stamp: 'Richo 12/28/2010 16:50'!
timingStatements
^ {
	#wait: -> 'waitv'
} as: Dictionary ! !

!NbcTranslator methodsFor: 'translating' stamp: 'Richo 12/29/2010 00:39'!
translate: aClass 
	variables := Set new.
	codeBlocks := aClass selectors collect: [:each| 
			self translateMethod: (aClass sourceCodeAt: each) named: each asString.
		].
	^ NbcProgram variables: variables codeBlocks: codeBlocks! !

!NbcTranslator methodsFor: 'translating' stamp: 'Richo 12/28/2010 17:47'!
translateMethod: methodSource named: methodName
statements := OrderedCollection new.
self visitNode: (RBParser parseMethod: methodSource).
^((methodName = 'main') ifTrue: [NbcThread] ifFalse: [NbcSubroutine])
	name: methodName statements: statements! !

RBProgramNodeVisitor subclass: #NxcTranslator
	instanceVariableNames: 'variableInfo source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

NxcTranslator subclass: #NxcPrimitiveRule
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Translation'!

!NxcPrimitiveRule class methodsFor: 'instance creation' stamp: 'Richo 1/1/2011 13:00'!
on: aByteString 
	^ (self basicNew setValue: aByteString) initialize ! !

!NxcPrimitiveRule methodsFor: 'applying' stamp: 'Richo 1/1/2011 13:04'!
applyTo: aMessageNode 
	| rule |
	rule := value.
	({aMessageNode receiver}, aMessageNode arguments)
		doWithIndex: [:node :index |
			source := WriteStream on: ''.
			self visitNode: node.
			rule := rule copyReplaceAll: 'arg' , (index - 1) asString with: source contents
			].
	^ rule! !

!NxcPrimitiveRule methodsFor: 'private' stamp: 'Richo 1/1/2011 13:00'!
setValue: aByteString 
	value := aByteString ! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:27'!
acceptAssignmentNode: anAssignmentNode
	self visitNode: anAssignmentNode variable.
	source space; nextPut: $=; space.
	self visitNode: anAssignmentNode value.
	source nextPut: $;; cr.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:11'!
acceptCascadeNode: aCascadeNode
	aCascadeNode messages do: [:each | self visitNode: each. source nextPut: $;; cr].! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:53'!
acceptLiteralNode: aLiteralNode
	source nextPutAll: aLiteralNode value asString.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:24'!
acceptMessageNode: aMessageNode 
(self isNxcPrimitive: aMessageNode selector)
	ifTrue: [
		source nextPutAll: ((self nxcPrimitives at: aMessageNode selector) asNxcRule applyTo: aMessageNode).
		^ self
		].
source nextPutAll: (aMessageNode selector asIdentifier: false); nextPut: $(.
aMessageNode arguments do: [:each | self visitNode: each] separatedBy: [source nextPut: $,; space].
source nextPut: $).! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 15:43'!
acceptMethodNode: methodNode
	| methodName |
	methodName := methodNode selector.
	methodName = #main
		ifTrue: [source nextPutAll: 'task']
		ifFalse: [source nextPutAll: (self returnTypeOf: methodNode)].
	source space; nextPutAll: (methodName asIdentifier: false); nextPut: $(.
	methodNode arguments do: [:arg | source nextPutAll: (variableInfo typeOf: arg name in: methodName); space; nextPutAll: arg name] separatedBy: [source nextPut: $,; space].
	source nextPut: $); cr; nextPut: ${; cr.
	methodNode allTemporaryVariables do: [:var |  source nextPutAll: (variableInfo typeOf: var in: methodName); space; nextPutAll: var; nextPut: $;; cr].
	source cr.
	self visitNode: methodNode body.
	source nextPut: $}.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:28'!
acceptPragmaNode: aPragmaNode
	"Do nothing"! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:29'!
acceptReturnNode: aReturnNode
	source nextPutAll: 'return'; space.
	self visitNode: aReturnNode value.
	source nextPut: $;; cr.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 13:21'!
acceptSequenceNode: aSequenceNode
aSequenceNode statements do: [:each | self visitNode: each. 
	each isMessage ifTrue: [source nextPut: $;; cr]].! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:28'!
acceptVariableNode: aVariableNode
	source nextPutAll: aVariableNode name! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 00:44'!
defaultVariableType
	^ #byte! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 02:39'!
extractVariablesFrom: aClass
variableInfo := VariableInfo new.
aClass instVarNames doWithIndex: [:varName :index |
	variableInfo instVars add: (varName -> (aClass instVarTypes at: index ifAbsent: [self defaultVariableType]))
	].
aClass selectors do: [:selector | | methodNode types |

	methodNode := (aClass parseTreeFor: selector).
	types := self getTypes: #argTypes: from: methodNode.
	methodNode allArgumentVariables doWithIndex: [:varName :index |
		(variableInfo at: selector) add: varName -> (types at: index ifAbsent: [self defaultVariableType])
		].
	types := self getTypes: #tempTypes: from: methodNode.
	methodNode allTemporaryVariables doWithIndex: [:varName :index |
		(variableInfo at: selector) add: varName -> (types at: index ifAbsent: [self defaultVariableType])
		].
	].
^ variableInfo
! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 00:28'!
getTypes: aSymbol from: aMethodNode 
| pragma |
pragma := aMethodNode pragmas detect: [:each | each selector = aSymbol] ifNone: [^#()].
^ (pragma arguments first value subStrings: {Character space}) collect: [:each | each asSymbol]! !

!NxcTranslator methodsFor: 'accessing' stamp: 'Richo 1/1/2011 04:15'!
isNxcPrimitive: selector
	^ self nxcPrimitives includesKey: selector! !

!NxcTranslator methodsFor: 'accessing' stamp: 'Richo 1/1/2011 14:08'!
nxcPrimitives
^ {
	#displayNumber:x:y: -> 'NumOut(arg2, arg3, arg1)'.
	#onFwd:speed: -> 'OnFwd(arg1, arg2)'.
	#+ -> '(arg0 + arg1)'.
	#- -> '(arg0 - arg1)'.
	#* -> '(arg0 * arg1)'.
	#= -> '(arg0 == arg1)'.
	#wait: -> 'Wait(arg1)'.
	#ifTrue:ifFalse: -> 'if(arg0){arg1}else{arg2}'.
	#ifTrue: -> 'if(arg0){arg1}'.
	#ifFalse: -> 'if(arg0){}else{arg1}'.
	} as: Dictionary! !

!NxcTranslator methodsFor: 'type info' stamp: 'Richo 1/1/2011 13:56'!
returnTypeOf: aMethodNode 
	| pragma |
	pragma := aMethodNode pragmas detect: [:each | each selector = #returnType:] ifNone: [^#void].
	^ pragma arguments first value asSymbol.
	! !

!NxcTranslator methodsFor: 'translating' stamp: 'Richo 1/1/2011 15:38'!
translate: aClass asProgramNamed: programName
	source := (WriteStream on: '').
	source nextPutAll: '/*'; space; nextPutAll: programName; space; nextPutAll: '*/'; cr; cr.
	self extractVariablesFrom: aClass.
	variableInfo instVars associationsDo: [:association | | name type |
		name := association key. 
		type := association value.
		source nextPutAll: type; space; nextPutAll: name; nextPut: $;; cr.
		].
	source cr.
	aClass selectors do: [:each | self visitNode: (aClass parseTreeFor: each)] separatedBy: [source cr; cr].
	^ NxcProgram name: programName source: source.! !

!NxcTranslator methodsFor: 'visiting' stamp: 'Richo 1/1/2011 03:25'!
visitArguments: aNodeCollection 
	"Do nothing"! !
